require_relative 'duplicable'

class Object
  ##
  # Returns a deep copy of object if it's duplicable. If it's
  # not duplicable, returns +self+.
  #
  #   object = Object.new
  #   dup    = object.mp_deep_dup
  #   dup.instance_variable_set(:@a, 1)
  #
  #   object.instance_variable_defined?(:@a) # => false
  #   dup.instance_variable_defined?(:@a)    # => true
  def mp_deep_dup
    mp_duplicable? ? dup : self
  end
end

class Array
  ##
  # Returns a deep copy of array.
  #
  #   array = [1, [2, 3]]
  #   dup   = array.mp_deep_dup
  #   dup[1][2] = 4
  #
  #   array[1][2] # => nil
  #   dup[1][2]   # => 4
  def mp_deep_dup
    map(&:mp_deep_dup)
  end
end

class Hash
  ##
  # Returns a deep copy of hash.
  #
  #   hash = { a: { b: 'b' } }
  #   dup  = hash.mp_deep_dup
  #   dup[:a][:c] = 'c'
  #
  #   hash[:a][:c] # => nil
  #   dup[:a][:c]  # => "c"
  def mp_deep_dup
    each_with_object(dup) do |(key, value), hash|
      hash.delete(key)
      hash[key.mp_deep_dup] = value.mp_deep_dup
    end
  end
end
